@@ -1,6 +1,6 @@
 stopwords <- {
     function(kind = "en") {
-        stopifnot(is.character(kind))
+        kind <- as.character(kind)
 
         resolved <- map_IETF_Snowball(kind)
         base <- if (is.na(resolved))
@@ -9,7 +9,8 @@ stopwords <- {
             "english"
         else
             resolved
-        s <- system.file("stopwords", paste(base, ".dat", sep = ""), package = "tm")
+        s <- system.file("stopwords", paste(base, ".dat", sep = ""),
+                         package = "tm")
         if (identical(s, ""))
             stop(paste("no stopwords available for '", base, "'", sep = ""))
         readLines(s, encoding = "UTF-8")
